(*  Rotation matrix utilities
	                       ver. 3.0                *)
(*                   written by Koun SHIRAI        *)

BeginPackage["KounPacks`RotationDif`"]

OutProduct::usage="OutProduct[a,b] makes the outproduct 
of two 3-dimensional vectors."

Distance::usage="Distance[p,q] gives the distance between
 the points p and q in a general vector space."

VectorAbs::usage="VectorAbs[v] gives the magnitude of the 
vector."

RotationNdir::usage="RotationNdir[alpha,v] creates the 
matrix of rotation of an angle alpha about the normarized 
vector v.  Hence, the v must be given so as to having 
unit length."

MatrixTrace::usage="MatrixTrace[R] gives the trace of a square matrix R.
This code is owing to the text of R. Maeder p.100."

SU2::usage="SU2[alpha,v] gives SU(2) matrixes represeting a rotation
 about the axis v by an angle alphs.  The v should be given so as to
  normarized."

(************ Body of the package.   **************)

Begin["`private`"]

OutProduct[a_?VectorQ, b_?VectorQ] := 
   {a[[2]]*b[[3]] - b[[2]]*a[[3]],
    a[[3]]*b[[1]] - b[[3]]*a[[1]], 
    a[[1]]*b[[2]] - b[[1]]*a[[2]]} /;
    	Length[a] == Length[b] == 3

OutProduct[a_?VectorQ, b_?VectorQ] := 
    (a[[1]]*b[[2]] - b[[1]]*a[[2]]) /;
    	Length[a] == Length[b] == 2

VectorAbs[v_?VectorQ]:=Sqrt[Cancel[Together[v.v]]]

Distance[p_?VectorQ,q_?VectorQ]:=Sqrt[(p-q).(p-q)] /;
					Length[p]==Length[q]

triCS = {Sin[ArcCos[x_]]:>Sqrt[1-x^2],
		Cos[ArcSin[x_]]:>Sqrt[1-x^2]}

MatrixTrace[m_?MatrixQ]:= Apply[Plus, Transpose[m, {1,1}]] 
		
RotationNdir[alpha_,{l_,m_,n_}]:=
	({{Cos[alpha]+l^2 (1-Cos[alpha]),
	l m (1-Cos[alpha]) - n Sin[alpha],
	n l (1-Cos[alpha]) + m Sin[alpha]},
	{m l (1-Cos[alpha]) + n Sin[alpha],
	Cos[alpha] + m^2 (1-Cos[alpha]),
	m n (1-Cos[alpha]) - l Sin[alpha]},
	{n l (1-Cos[alpha]) - m Sin[alpha],
	n m (1-Cos[alpha]) + l Sin[alpha],
	Cos[alpha] + n^2 (1-Cos[alpha])}} /.triCS) /;
	N[l^2+m^2+n^2]==1.
	(* This is used from Problem 2.5.2 in Lax. *)
	
SU2[alpha_,{l_,m_,n_}]:=
	({{Cos[alpha/2] - I n Sin[alpha/2],
	-(m + I l) Sin[alpha/2]},
	{(m - I l) Sin[alpha/2],
	Cos[alpha/2] + I n Sin[alpha/2]}} /.triCS) /;
	N[l^2+m^2+n^2]==1.
	(* This is given from Eq.2.6.7 in Lax. *)

End[]

EndPackage[]