(*^

::[paletteColors = 128; 
	fontset = title, "Times", 24, L2, bold, nohscroll;
	fontset = subtitle, "Times", 18, L2, center, bold, nohscroll;
	fontset = subsubtitle, "Times", 14, L2, center, bold, nohscroll;
	fontset = section, "Helvetica", 14, L2, bold, nohscroll, grayBox;
	fontset = subsection, "Helvetica", 12, L2, bold, nohscroll, blackBox;
	fontset = subsubsection, "Helvetica", 10, L2, bold, nohscroll, whiteBox;
	fontset = text, "Times", 12, L2, nohscroll;
	fontset = smalltext, "Times", 10, L2, nohscroll;
	fontset = input, "Courier", 12, L2, bold, nowordwrap;
	fontset = output, "Courier", 10, L2, nowordwrap;
	fontset = message, "Courier", 12, L2, R65535, nowordwrap;
	fontset = print, "Courier", 12, L2, nowordwrap;
	fontset = info, "Courier", 12, L2, nowordwrap;
	fontset = postscript, "Courier", 12, L2, nowordwrap;
	fontset = name, "Geneva", 10, L2, italic, B65535, nowordwrap, nohscroll;
	fontset = header, "Times", 10, L2;
	fontset = footer, "Times", 12, L2, center;
	fontset = help, "Geneva", 10, L2, nohscroll;
	fontset = clipboard, "New York", 12, L2;
	fontset = completions, "New York", 12, L2, nowordwrap;
	fontset = network, "Courier", 10, L2, nowordwrap;
	fontset = graphlabel, "Courier", 12, L2, nowordwrap;
	fontset = special1, "New York", 12, L2, nowordwrap;
	fontset = special2, "New York", 12, L2, center, nowordwrap;
	fontset = special3, "New York", 12, L2, right, nowordwrap;
	fontset = special4, "New York", 12, L2, nowordwrap;
	fontset = special5, "New York", 12, L2, nowordwrap;]
:[font = input; initialization; ]
*)
(* This package includes several functions concerning
   matrix algebra.
   Some functions in this package should be used under
    rather restricted conditions.   *)
(*                         ver. 2.7                *)
(*                   written by Koun SHIRAI        *)

BeginPackage["RotationDif`"]

OutProduct::usage="OutProduct[a,b] makes the outproduct 
of two 3-dimensional vectors."

Distance::usage="Distance[p,q] gives the distance between
 the points p and q in a general vector space."

VectorAbs::usage="VectorAbs[v] gives the magnitude of the 
vector."

rRotated::usage="rRotated[r,alpha,n] sends a vector r to
the new vector r' rotated about the axis n by the angle 
alpha."

RotationNaxis::usage="RotationNaxis[phi,theta,psi] represents
 the matrix of rotation psi about the axis n, whose direction 
 is presented by (phi, theta)." 

RotationNdir::usage="RotationNdir[alpha,v] creates the 
matrix of rotation of an angle alpha about the normarized 
vector v.  Hence, the v must be given so as to having 
unit length."

RotationEular::usage="RotationEular[alpha,beta,gamma] represents
 the matrix of rotation with respect to the Eulerian angles."

MatrixTrace::usage="MatrixTrace[R] gives the trace of a square matrix R.
This code is owing to the text of R. Maeder p.100."

SU2::usage="SU2[alpha,v] gives SU(2) matrixes represeting a rotation
 about the axis v by an angle alphs.  The v should be given so as to
  normarized."

(************ Body of the package.   **************)

Begin["`private`"]

OutProduct[a_?VectorQ, b_?VectorQ] := 
   {a[[2]]*b[[3]] - b[[2]]*a[[3]],
    a[[3]]*b[[1]] - b[[3]]*a[[1]], 
    a[[1]]*b[[2]] - b[[1]]*a[[2]]} /; 
																			    Length[a] == Length[b] == 3

Distance[p_?VectorQ,q_?VectorQ]:=Sqrt[(p-q).(p-q)] /;
																									Length[p]==Length[q]

VectorAbs[v_?VectorQ]:=Sqrt[Cancel[Together[v.v]]]

triCS = {Sin[ArcCos[x_]]:>Sqrt[1-x^2],
									Cos[ArcSin[x_]]:>Sqrt[1-x^2]}
		
rRotated[r_?VectorQ,alpha_,n_?VectorQ]:= 
	Cos[alpha] r + (n.r) (1-Cos[alpha]) n + 
	Sin[alpha] OutProduct[n,r] //. triCS /; 
			Dimensions[r]==Dimensions[n]
								
RotationNaxis[phi_,theta_,psi_]:={
 {Cos[psi] + Cos[phi]^2*(1 - Cos[psi])*Sin[theta]^2, 
		Cos[theta]*Sin[psi] + (1 - Cos[psi])*Sin[phi]*Cos[phi]*
			Sin[theta]^2,
		Cos[phi]*(1 - Cos[psi])*Cos[theta]*Sin[theta] - 
			Sin[phi]*Sin[psi]*Sin[theta]}, 
 {-Cos[theta]*Sin[psi] + Cos[phi]*(1 - Cos[psi])*Sin[phi]*
			Sin[theta]^2,
		Cos[psi] + (1 - Cos[psi])*Sin[phi]^2*Sin[theta]^2, 
		(1 - Cos[psi])*Cos[theta]*Sin[phi]*Sin[theta] + 
			Cos[phi]*Sin[psi]*Sin[theta]}, 
 {(1 - Cos[psi])*Cos[theta]*Cos[phi]*Sin[theta] + 
			Sin[phi]*Sin[psi]*Sin[theta], 
		(1 - Cos[psi])*Cos[theta]*Sin[phi]*Sin[theta] - 
			Cos[phi]*Sin[psi]*Sin[theta], 
		Cos[psi] + (1 - Cos[psi])*Cos[theta]^2}
	} //. triCS

RotationNdir[alpha_,v_?VectorQ]:=
   {	{Cos[alpha]+v[[1]]^2 (1-Cos[alpha]),
	 v[[2]] v[[1]] (1-Cos[alpha])+v[[3]] Sin[alpha],
	 v[[3]] v[[1]] (1-Cos[alpha])-v[[2]] Sin[alpha]},
	{v[[1]] v[[2]] (1-Cos[alpha])-v[[3]] Sin[alpha],
	 Cos[alpha]+v[[2]]^2 (1-Cos[alpha]),
	 v[[3]] v[[2]] (1-Cos[alpha])+v[[1]] Sin[alpha]},
	{v[[1]] v[[3]] (1-Cos[alpha])+ v[[2]] Sin[alpha],
	 v[[2]] v[[3]] (1-Cos[alpha])-v[[1]] Sin[alpha],
	 Cos[alpha]+v[[3]]^2(1-Cos[alpha])} } //. triCS /;
						N[VectorAbs[v]]==1.
					  (* This is the transpose of Prob.2.5.2,
					    but the same as Eq. 2.5.11 in Lax. *)    


(* Constract the ratation matrixes represented by Eulerian angles. *)
r11[alpha_,beta_,gamma_]:=-Sin[alpha] Sin[gamma]+
						Cos[beta] Cos[alpha] Cos[gamma] //. triCS;
r12[alpha_,beta_,gamma_]:=Cos[alpha] Sin[gamma]+
						Cos[beta] Sin[alpha] Cos[gamma] //. triCS;
r13[alpha_,beta_,gamma_]:=-Sin[beta] Cos[gamma] //. triCS;
r21[alpha_,beta_,gamma_]:=-Sin[alpha] Cos[gamma]-
						Cos[beta] Cos[alpha] Sin[gamma] //. triCS;
r22[alpha_,beta_,gamma_]:=Cos[alpha] Cos[gamma]-
						Cos[beta] Sin[alpha] Sin[gamma] //. triCS;
r23[alpha_,beta_,gamma_]:=Sin[beta] Sin[gamma] //. triCS;
r31[alpha_,beta_,gamma_]:=Sin[beta] Cos[alpha] //. triCS;
r32[alpha_,beta_,gamma_]:=Sin[beta] Sin[alpha] //. triCS;
r33[alpha_,beta_,gamma_]:=Cos[beta] //. triCS

r1[alpha_,beta_,gamma_]:={r11[alpha,beta,gamma],
					r12[alpha,beta,gamma],r13[alpha,beta,gamma]}
r2[alpha_,beta_,gamma_]:={r21[alpha,beta,gamma],
					r22[alpha,beta,gamma],r23[alpha,beta,gamma]}
r3[alpha_,beta_,gamma_]:={r31[alpha,beta,gamma],
					r32[alpha,beta,gamma],r33[alpha,beta,gamma]}

RotationEular[alpha_,beta_,gamma_]:={r1[alpha,beta,gamma],
				r2[alpha,beta,gamma],r3[alpha,beta,gamma]}

MatrixTrace[m_?MatrixQ]:= Apply[Plus, Transpose[m, {1,1}]] 

SU2[alpha_,v_?VectorQ]:=
			{{Cos[alpha/2] + I v[[3]] Sin[alpha/2],
			 v[[2]] Sin[alpha/2] + I v[[1]] Sin[alpha/2]},
			{-v[[2]] Sin[alpha/2] + I v[[1]] Sin[alpha/2],
			 Cos[alpha/2] - I v[[3]] Sin[alpha/2]}} /;
						N[VectorAbs[v]]==1.
					  (* This is the transpose of Eq.2.6.7 in Lax. *)    
End[]

EndPackage[]
(*
;[s]
5:0,0;3309,1;3314,0;4859,1;4864,0;4938,-1;
2:3,14,10,Courier,1,12,0,0,0;2,13,9,Courier,1,10,0,0,0;
^*)