(*^

::[paletteColors = 128; 
	fontset = title, "Times", 24, L2, bold, nohscroll;
	fontset = subtitle, "Times", 18, L2, center, bold, nohscroll;
	fontset = subsubtitle, "Times", 14, L2, center, bold, nohscroll;
	fontset = section, "Helvetica", 14, L2, bold, nohscroll, grayBox;
	fontset = subsection, "Helvetica", 12, L2, bold, nohscroll, blackBox;
	fontset = subsubsection, "Helvetica", 10, L2, bold, nohscroll, whiteBox;
	fontset = text, "Courier", 12, L2, nohscroll;
	fontset = smalltext, "Helvetica", 10, L2, nohscroll;
	fontset = input, "Courier", 12, L2, bold, nowordwrap;
	fontset = output, "Courier", 10, L2, nowordwrap;
	fontset = message, "Courier", 12, L2, R65535, nowordwrap;
	fontset = print, "Courier", 12, L2, nowordwrap;
	fontset = info, "Courier", 12, L2, nowordwrap;
	fontset = postscript, "Courier", 12, L2, nowordwrap;
	fontset = name, "Geneva", 10, L2, italic, B65535, nowordwrap, nohscroll;
	fontset = header, "Times", 10, L2;
	fontset = footer, "Courier", 10, L2, center;
	fontset = help, "Geneva", 10, L2, nohscroll;
	fontset = clipboard, "New York", 12, L2;
	fontset = completions, "New York", 12, L2, nowordwrap;
	fontset = network, "Courier", 10, L2, nowordwrap;
	fontset = graphlabel, "Courier", 12, L2, nowordwrap;
	fontset = special1, "New York", 12, L2, nowordwrap;
	fontset = special2, "New York", 12, L2, center, nowordwrap;
	fontset = special3, "New York", 12, L2, right, nowordwrap;
	fontset = special4, "New York", 12, L2, nowordwrap;
	fontset = special5, "New York", 12, L2, nowordwrap;]
:[font = input; initialization; ]
*)
(*
	Read in this file to obtain rules that allow expressions involving
	real and imaginary parts to simplify more often.
	
	This version is one adding further objects to 
	the original ReIm package to make it easy 
		treating spinors functions.
			Rev.2.5
			modefied by K. SHIRAI
*)


BeginPackage["Algebra`ReIm`"]

TagReal::usage="TagReal[x] sets x to a real number."

Begin["`Private`"]

TagReal[x_]:=(x/:Re[x]=x; x/:Im[x]=0;)
SetAttributes[TagReal,{Listable, Orderless, OneIdentity}]
Protect[TagReal]

Unprotect[Re,Im, Abs, Conjugate, RealQ]

Re[x_]:=x /; Im[x]==0

Re[x_+y_] := Re[x] + Re[y]
Im[x_+y_] := Im[x] + Im[y]

Re[x_ y_] := Re[x] Re[y] - Im[x] Im[y]
Im[x_ y_] := Re[x] Im[y] + Im[x] Re[y]

Re[ 1 / x_ ] := Re[x] / (Re[x]^2 + Im[x]^2)
Im[ 1 / x_ ] := -Im[x] / (Re[x]^2 + Im[x]^2)

Re[E^x_] := Cos[Im[x]] Exp[Re[x]]
Im[E^x_] := Sin[Im[x]] Exp[Re[x]]

Im[x_ ^ 2] := 2 Re[x] Im[x]

Re[ x_ ^ n_Integer ] := Block[{a, b},
	a = Round[n/2];
	b = n-a;
	Re[x^a] Re[x^b] - Im[x^a] Im[x^b]
	]

Im[ x_ ^ n_Integer ] := Block[{a, b},
	a = Round[n/2];
	b = n-a;
	Re[x^a] Im[x^b] + Im[x^a] Re[x^b]
	]

Re[x_Integer ^ n_Rational] := 0    /; IntegerQ[2n] && x<0
Im[x_Integer ^ n_Rational] := 
	(-x) ^ n   (-1) ^ ((Numerator[n]-1)/2) /; IntegerQ[2n] && x<0

Re[x_Integer ^ n_Rational] := x^n /; OddQ[Denominator[n]] || x>0
Im[x_Integer ^ n_Rational] := 0   /; OddQ[Denominator[n]] || x>0

Re[(-1) ^ n_Rational] := Cos[n Pi]
Im[(-1) ^ n_Rational] := Sin[n Pi]

Re[Sin[x_]] := Sin[Re[x]] Cosh[Im[x]]
Im[Sin[x_]] := Cos[Re[x]] Sinh[Im[x]]

Re[Log[r_?Positive]] := Log[r]
Im[Log[r_?Positive]] := 0
Re[Log[r_?Negative]] := Log[-r]
Im[Log[r_?Negative]] := Pi

Re[Log[z_]] := (1/2) Log[Re[z]^2 + Im[z]^2]
Im[Log[z_]] := Arg[z]

Re[Log[a_ b_]] := Re[Log[a] + Log[b]]
Im[Log[a_ b_]] := Im[Log[a] + Log[b]]

Re[Cos[x_]] :=  Cos[Re[x]] Cosh[Im[x]]
Im[Cos[x_]] := -Sin[Re[x]] Sinh[Im[x]]

Re[Tan[x_]] := Re[Sin[x]/Cos[x]]
Im[Tan[x_]] := Im[Sin[x]/Cos[x]]

Re[Conjugate[z_]] :=  Re[z]
Im[Conjugate[z_]] := -Im[z]

Re[Sqrt[x_]] := Sqrt[x] /;Positive[N[x]]==True
Im[Sqrt[x_]] := 0 /;Positive[N[x]]==True

Re[1/Sqrt[x_]] := 1/Sqrt[x] /;Positive[N[x]]==True
Im[1/Sqrt[x_]] := 0 /;Positive[N[x]]==True

Conjugate[Re[x_]] := Re[x]
Conjugate[Im[x_]] := Im[x]

RealQ[Re[x_]] := True
RealQ[Im[x_]] := True

Conjugate[x_] := x  /;Re[x]==x&&Im[x]==0
Conjugate[I x_]:= -I x  /;Re[x]==x&&Im[x]==0	

Conjugate[x_Real] := x
Conjugate[I x_Real]:= - x	
Re[Abs[z_]] := Abs[z]
Im[Abs[z_]] := 0

RealQ[E^x_Real]:= True
RealQ[Sin[x_Real]]:= True


RealQ[Cos[x_Real]]:= True


Re[Re[z_]] := Re[z]
Re[Im[z_]] := Im[z]

Im[Re[z_]] := 0
Im[Im[z_]] := 0

Conjugate[x_+y_]:= Conjugate[x]+Conjugate[y]
Conjugate[x_ y_]:= Conjugate[x] Conjugate[y]
Conjugate[1/x_]:= 1/Conjugate[x]
Conjugate[x_^n_Integer]:= Conjugate[x]^n
Conjugate[Conjugate[x_]]:= x
Conjugate[E^x_Real]:= E^x
Conjugate[Sin[x_Real]]:= Sin[x]
Conjugate[Cos[x_Real]]:= Cos[x]
Conjugate[E^Re[x_]]:= E^Re[x]
Conjugate[E^Im[x_]]:= E^Im[x]
Conjugate[Sin[Re[x_]]]:= Sin[Re[x]]
Conjugate[Cos[Re[x_]]]:= Cos[Re[x]]
Conjugate[Sin[Im[x_]]] := Sin[Im[x]]
Conjugate[Cos[Im[x_]]] := Cos[Im[x]]
Conjugate[E^Im[x_]] := E^Im[x]
Protect[Re,Im, Abs, Conjugate, RealQ]

End[]
EndPackage[]

Null
(*
;[s]
1:0,0;3195,-1;
1:1,14,10,Courier,0,12,0,0,0;
^*)