(* Simplications of SQUARE ROOT functions 
		This package is not intended to use for general purposees.
			How to simplify Sqrt functions is depending on 
					what you want to do.		           *)

(* 		
	     	Rev. 3.0
	            						by Koun SHIRAI *)


BeginPackage["Algebra`SquareRoot`"]

SqrtFactor::usage = "SqrtFactor[expr] is a function factoring
sqrt functions."

SqrtCanonical::usage = "SqrtCanonical[expr] applies basic square root
	simplifications to expressions."

SqrtOpen::usege = "SqrtOpen[expr] acts on square root functions
for factoring."

SqrtRationize::usege = "SqrtRationize[expr] is used for rationalizaton
of sqrt fuctions."


Begin["`Private`"]

SqrtFactor[e_] :=  Cancel[(e //. 
	a_. x_ + b_. x_ :>
		(a+b) x /; NumberQ[N[a]]&&NumberQ[N[b]] ) //.
	{a_. Sqrt[x_Integer]+b_. x_Integer^p_Rational :> 
		Sqrt[x](a + b x^(p-(1/2))) /;
			IntegerQ[a]&&IntegerQ[b]&&OddQ[2 p]&&p>1,
	a_. x_Integer^p_Rational :>
			(a x^(p-(1/2))) Sqrt[x] /;
				IntegerQ[a]&&p>1 } ] //.
	{a_Integer + b_Integer Sqrt[c_Integer]  :>
		GCD[a,b]((a/GCD[a,b])+(b/GCD[a,b]) Sqrt[c]) }


SqrtRationizeRel ={
	1/Sqrt[x_Integer] :> Sqrt[x]/x,
	1/(Sqrt[x_Integer] Sqrt[y_Integer]) :> 1/Sqrt[x y],
	1/(a_Integer+b_. Sqrt[x_Integer]) :> (a-b Sqrt[x])/(a^2-b^2 x) /; 
			IntegerQ[b],
	1/Sqrt[a_Integer+b_. Sqrt[x_Integer]] :> 
		Sqrt[a+b Sqrt[x]](a-b Sqrt[x])/(a^2-x b^2) /; 
			IntegerQ[b]
}

SqrtRationizeRel = Dispatch[SqrtRationizeRel]
Protect[SqrtRationizeRel]

SqrtRationize[e_] := e /. SqrtRationizeRel


SqrtCanonicalRel = {

	Sqrt[x_Integer] Sqrt[y_Integer] :> Sqrt[x y],

	(a_.+b_. Sqrt[x_Integer])(c_.+d_. Sqrt[y_Integer]) :> 
		a c + b d Sqrt[x y] + a d Sqrt[y] + b c Sqrt[x] /;
			IntegerQ[a]&&IntegerQ[b]&&IntegerQ[c]&&
			IntegerQ[d],
	
	Sqrt[a_Integer] Sqrt[b_Integer + c_. Sqrt[x_Integer]] :>
		Sqrt[a b + a c Sqrt[x]] /; IntegerQ[c],
	Sqrt[a_Integer+b_. Sqrt[x_Integer]] (c_Integer+d_. Sqrt[y_Integer]) :>
		Sign[N[c+d Sqrt[y]]] Sqrt[Expand[(a+b Sqrt[x])(c+d Sqrt[y])^2]] /;
			IntegerQ[b]&&IntegerQ[d],
	Sqrt[a_Integer+b_. Sqrt[x_Integer]] Sqrt[c_Integer+d_. Sqrt[y_Integer]] :>
		Sqrt[a c + b d Sqrt[x y] + a d Sqrt[y] + b c Sqrt[x]] /;
			IntegerQ[b]&&IntegerQ[d],
	Sqrt[a_Integer+b_. Sqrt[x_Integer]] (c_. Sqrt[y_Integer]+
		d_. Sqrt[z_Integer]) :>
			Sign[N[c Sqrt[y]+d Sqrt[z]]] 
			Sqrt[Expand[(a+b Sqrt[x]) (c Sqrt[y]+d Sqrt[z])^2] ] /;
				IntegerQ[b]&&IntegerQ[c]&&IntegerQ[d],
	Sqrt[x_+Sqrt[x_]]/Sqrt[x_-Sqrt[x_]] :> 
		Sqrt[x-1]/(Sqrt[x]-1) /; 
			IntegerQ[x]&&x>1,
	Sqrt[x_-Sqrt[x_]]/Sqrt[x_+Sqrt[x_]] :> 
		(Sqrt[x]-1)/Sqrt[x-1] /; 
			IntegerQ[x]&&x>1,
	(a_Integer+b_. Sqrt[x_Integer])(c_. + d_. Sqrt[y_Integer] +
		e_. Sqrt[f_Integer+g_. Sqrt[z_Integer]])  :>
			a c+(a d Sqrt[y] + b c Sqrt[x] + b d Sqrt[x y]) +
			a e Sqrt[f+g Sqrt[z]]+b e Sqrt[x f+ x g Sqrt[z]] /;
				IntegerQ[b]&&IntegerQ[d]&&IntegerQ[e]&&IntegerQ[g]
			

}

SqrtCanonicalRel = Dispatch[SqrtCanonicalRel]
Protect[SqrtCanonicalRel]

SqrtCanonical[e_] := Cancel[ SqrtFactor[e] /.
						SqrtRationizeRel //.SqrtCanonicalRel ]


SqrtOpenRel = 
{
	Sqrt[a_Integer + b_Integer Sqrt[x_]] :> 
		Sqrt[ GCD[a,b](Cancel[a/GCD[a,b]]+
		Cancel[b/GCD[a,b]] Sqrt[x])],
	Sqrt[a_Integer+Sqrt[x_Integer]] :>
		Sqrt[2](1+Sqrt[x])/2 /;
			2a==1+x,
	Sqrt[a_Integer-Sqrt[x_Integer]] :>
		Sqrt[2](-1+Sqrt[x])/2 /;
			2a==1+x,
	Sqrt[2]Sqrt[a_Integer+Sqrt[x_Integer]] :>
		1+Sqrt[x] /;
			2a==1+x,
	Sqrt[2]Sqrt[a_Integer-Sqrt[x_Integer]] :>
		-1+Sqrt[x] /;
			2a==1+x,
	Sqrt[a_Integer]Sqrt[b_Integer+Sqrt[x_Integer]] :> 
		Together[ Expand[Sqrt[a/2](1 + Sqrt[x])] ] /;
				2 b==1+x,
	Sqrt[a_Integer]Sqrt[b_Integer-Sqrt[x_Integer]] :> 
		Together[ Expand[Sqrt[a/2](-1 + Sqrt[x])] ] /;
				2 b==1+x,
	Sqrt[a_Integer]Sqrt[b_Integer+Sqrt[x_Integer]] :> 
		(a/2) + Sqrt[x] /;
				EvenQ[a]&&(b==(a/4)+(x/a)),	
	Sqrt[a_Integer]Sqrt[b_Integer-Sqrt[x_Integer]] :> 
		(a/2) - Sqrt[x] /;
				EvenQ[a]&&(b==(a/4)+(x/a))&&
				((a/2)-Sqrt[x]>0),	
	Sqrt[a_Integer]Sqrt[b_Integer+Sqrt[x_Integer]] :> 
		(a/2) + Sqrt[x] /;
				EvenQ[a]&&(b==(a/4)+(x/a)),	
	Sqrt[a_Integer]Sqrt[b_Integer-Sqrt[x_Integer]] :> 
		(a/2) - Sqrt[x] /;
				EvenQ[a]&&(b==(a/4)+(x/a))&&
				((a/2)-Sqrt[x]>0),	
	Sqrt[a_Integer]Sqrt[b_Integer+
	c_Integer?Positive Sqrt[x_Integer]] :> 
		Sqrt[a/2]((a/2) + Sqrt[x]) /;
				(2 b==(a/2)^2+x)&&c==a/2,	
	Sqrt[a_Integer]Sqrt[b_Integer+
	c_Integer?Negative Sqrt[x_Integer]] :> 
		Sqrt[a/2]((a/2) - Sqrt[x]) /;
				(2 b==(a/2)^2+x)&&c==-a/2	

}

SqrtOpenRel = Dispatch[SqrtOpenRel]

Protect[SqrtOpenRel]

SqrtOpen[e_] := Cancel[e //. SqrtOpenRel]

End[]

Protect[SqrtFactor, SqrtRationize, SqrtCanonical, SqrtOpen]

EndPackage[]
